@@ -2,4 +2,9 @@
 
 import client
 reload(client)
+
+import utils
+reload(utils)
+
 from client import *  # NOQA
+from utils import *  # NOQA
