@@ -4,15 +4,14 @@ import unittest
 import time
 realTimeTime = time.time
 import datetime
-import uuid
 
-import httmock
 import mock
 import requests
 
 import base
 import taskcluster.client as subject
 import taskcluster.exceptions as exc
+import taskcluster.utils as utils
 
 
 class ClientTest(base.TCTest):
@@ -49,13 +48,13 @@ class ClientTest(base.TCTest):
 
 class TestSubArgsInRoute(ClientTest):
   def test_valid_no_subs(self):
-    provided = '/no/args/here'
+    provided = {'route': '/no/args/here', 'name': 'test'}
     expected = 'no/args/here'
     result = self.client._subArgsInRoute(provided, {})
     self.assertEqual(expected, result)
 
   def test_valid_one_sub(self):
-    provided = '/one/<argToSub>/here'
+    provided = {'route': '/one/<argToSub>/here', 'name': 'test'}
     expected = 'one/value/here'
     arguments = {'argToSub': 'value'}
     result = self.client._subArgsInRoute(provided, arguments)
@@ -63,102 +62,71 @@ class TestSubArgsInRoute(ClientTest):
 
   def test_invalid_one_sub(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._subArgsInRoute('/one/<argToSub>/here', {'unused': 'value'})
+      self.client._subArgsInRoute({'route': '/one/<argToSub>/here', 'name': 'test'}, {'unused': 'value'})
 
   def test_invalid_route_no_sub(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._subArgsInRoute('adjfjlaksdfjs', {'should': 'fail'})
+      self.client._subArgsInRoute({'route': 'askldjflkasdf', 'name': 'test'}, {'should': 'fail'})
 
   def test_invalid_route_no_arg(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._subArgsInRoute('adjfjlaksdfjs', {'should': 'fail'})
+      self.client._subArgsInRoute({'route': 'askldjflkasdf', 'name': 'test'}, {'should': 'fail'})
 
 
 class TestProcessArgs(ClientTest):
   def test_no_args(self):
-    self.assertEqual({}, self.client._processArgs([]))
+    self.assertEqual({}, self.client._processArgs({'args': [], 'name': 'test'}))
 
   def test_positional_args_only(self):
     expected = {'test': 'works', 'test2': 'still works'}
-    actual = self.client._processArgs(['test', 'test2'], 'works', 'still works')
+    entry = {'args': ['test', 'test2'], 'name': 'test'}
+    actual = self.client._processArgs(entry, 'works', 'still works')
     self.assertEqual(expected, actual)
 
   def test_keyword_args_only(self):
     expected = {'test': 'works', 'test2': 'still works'}
-    actual = self.client._processArgs(['test', 'test2'], test2='still works', test='works')
+    entry = {'args': ['test', 'test2'], 'name': 'test'}
+    actual = self.client._processArgs(entry, test2='still works', test='works')
     self.assertEqual(expected, actual)
 
-  def test_keyword_overwrites_positional(self):
-    expected = {'test': 'works'}
-    actual = self.client._processArgs(['test'], 'broken', test='works')
+  def test_int_args(self):
+    expected = {'test': 'works', 'test2': 42}
+    entry = {'args': ['test', 'test2'], 'name': 'test'}
+    actual = self.client._processArgs(entry, 'works', 42)
     self.assertEqual(expected, actual)
 
+  def test_keyword_and_positional(self):
+    entry = {'args': ['test'], 'name': 'test'}
+    with self.assertRaises(exc.TaskclusterFailure):
+      self.client._processArgs(entry, 'broken', test='works')
+
   def test_invalid_not_enough_args(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['test'])
+      self.client._processArgs({'args': ['test'], 'name': 'test'})
 
   def test_invalid_too_many_positional_args(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['test'], 'enough', 'one too many')
+      self.client._processArgs({'args': ['test'], 'name': 'test'}, 'enough', 'one too many')
 
   def test_invalid_too_many_keyword_args(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['test'], test='enough', test2='one too many')
+      self.client._processArgs({'args': ['test'], 'name': 'test'}, test='enough', test2='one too many')
 
   def test_invalid_missing_arg_positional(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['test', 'test2'], 'enough')
+      self.client._processArgs({'args': ['test', 'test2'], 'name': 'test'}, 'enough')
 
   def test_invalid_not_enough_args_because_of_overwriting(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['test', 'test2'], 'enough', test='enough')
+      self.client._processArgs({'args': ['test', 'test2'], 'name': 'test'}, 'enough', test='enough')
 
   def test_invalid_positional_not_string_empty_dict(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['a'], {})
+      self.client._processArgs({'args': ['test'], 'name': 'test'}, {})
 
   def test_invalid_positional_not_string_non_empty_dict(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['a'], {'john': 'ford'})
-
-  def test_invalid_positional_not_string_int(self):
-    with self.assertRaises(exc.TaskclusterFailure):
-      self.client._processArgs(['a'], 4)
-
-
-class TestMakeSingleHttpRequest(ClientTest):
-  def test_success_no_payload(self):
-    @httmock.all_requests
-    def response_content(url, request):
-      return {'status_code': 200, 'content': {}}
-
-    with httmock.HTTMock(response_content):
-      d = self.client._makeSingleHttpRequest('GET', 'http://www.example.com', {})
-      self.assertEqual(d.json(), {})
-      self.assertEqual(d.status_code, 200)
-      d.raise_for_status()
-
-  def test_success_payload(self):
-    @httmock.all_requests
-    def response_content(url, request):
-      self.assertEqual(request.body, '{"i": "j"}')
-      return {'status_code': 200, 'content': {'k': 'l'}}
-
-    with httmock.HTTMock(response_content):
-      d = self.client._makeSingleHttpRequest('GET', 'http://www.example.com', {'i': 'j'})
-      self.assertEqual(d.json(), {'k': 'l'})
-      self.assertEqual(d.status_code, 200)
-      d.raise_for_status()
-
-  def test_failure(self):
-    @httmock.all_requests
-    def response_content(url, requet):
-      return {'status_code': 404}
-
-    with httmock.HTTMock(response_content):
-      d = self.client._makeSingleHttpRequest('GET', 'http://www.example.com', {})
-      with self.assertRaises(requests.exceptions.RequestException):
-        d.raise_for_status()
+      self.client._processArgs({'args': ['test'], 'name': 'test'}, {'john': 'ford'})
 
 
 # This could probably be done better with Mock
@@ -181,16 +149,16 @@ class TestMakeHttpRequest(ClientTest):
     ClientTest.setUp(self)
 
   def test_success_first_try(self):
-    with mock.patch.object(self.client, '_makeSingleHttpRequest') as p:
+    with mock.patch.object(utils, 'makeSingleHttpRequest') as p:
       expected = {'test': 'works'}
       p.return_value = ObjWithDotJson(200, expected)
 
       v = self.client._makeHttpRequest('GET', 'http://www.example.com', {})
-      p.assert_called_once_with('GET', 'http://www.example.com', {}, 'e30=')
+      p.assert_called_once_with('GET', 'http://www.example.com', {}, mock.ANY)
       self.assertEqual(expected, v)
 
   def test_success_fifth_try_status_code(self):
-    with mock.patch.object(self.client, '_makeSingleHttpRequest') as p:
+    with mock.patch.object(utils, 'makeSingleHttpRequest') as p:
       expected = {'test': 'works'}
       sideEffect = [
         ObjWithDotJson(500, None),
@@ -200,7 +168,7 @@ class TestMakeHttpRequest(ClientTest):
         ObjWithDotJson(200, expected)
       ]
       p.side_effect = sideEffect
-      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, 'e30=')
+      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, mock.ANY)
                        for x in range(self.client.options['maxRetries'])]
 
       v = self.client._makeHttpRequest('GET', 'http://www.example.com', {})
@@ -208,7 +176,7 @@ class TestMakeHttpRequest(ClientTest):
       self.assertEqual(expected, v)
 
   def test_success_fifth_try_connection_errors(self):
-    with mock.patch.object(self.client, '_makeSingleHttpRequest') as p:
+    with mock.patch.object(utils, 'makeSingleHttpRequest') as p:
       expected = {'test': 'works'}
       sideEffect = [
         requests.exceptions.RequestException,
@@ -218,7 +186,7 @@ class TestMakeHttpRequest(ClientTest):
         ObjWithDotJson(200, expected)
       ]
       p.side_effect = sideEffect
-      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, 'e30=')
+      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, mock.ANY)
                        for x in range(self.client.options['maxRetries'])]
 
       v = self.client._makeHttpRequest('GET', 'http://www.example.com', {})
@@ -226,18 +194,18 @@ class TestMakeHttpRequest(ClientTest):
       self.assertEqual(expected, v)
 
   def test_failure_status_code(self):
-    with mock.patch.object(self.client, '_makeSingleHttpRequest') as p:
+    with mock.patch.object(utils, 'makeSingleHttpRequest') as p:
       p.return_value = ObjWithDotJson(500, None)
-      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, 'e30=')
+      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, mock.ANY)
                        for x in range(self.client.options['maxRetries'])]
       with self.assertRaises(exc.TaskclusterRestFailure):
         self.client._makeHttpRequest('GET', 'http://www.example.com', {})
       p.assert_has_calls(expectedCalls)
 
   def test_failure_connection_errors(self):
-    with mock.patch.object(self.client, '_makeSingleHttpRequest') as p:
+    with mock.patch.object(utils, 'makeSingleHttpRequest') as p:
       p.side_effect = requests.exceptions.RequestException
-      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, 'e30=')
+      expectedCalls = [mock.call('GET', 'http://www.example.com', {}, mock.ANY)
                        for x in range(self.client.options['maxRetries'])]
       with self.assertRaises(exc.TaskclusterRestFailure):
         self.client._makeHttpRequest('GET', 'http://www.example.com', {})
@@ -296,7 +264,7 @@ class TestMakeApiCall(ClientTest):
     with mock.patch.object(self.client, '_makeHttpRequest') as patcher:
       patcher.return_value = expected
 
-      actual = self.client.two_args_no_input('argone', arg1='argtwo')
+      actual = self.client.two_args_no_input('argone', 'argtwo')
       self.assertEqual(expected, actual)
 
       patcher.assert_called_once_with('get', 'two_args_no_input/argone/argtwo', None)
@@ -306,20 +274,20 @@ class TestMakeApiCall(ClientTest):
     with mock.patch.object(self.client, '_makeHttpRequest') as patcher:
       patcher.return_value = expected
 
-      actual = self.client.no_args_with_input()
+      actual = self.client.no_args_with_input({})
       self.assertEqual(expected, actual)
 
-      patcher.assert_called_once_with('get', 'no_args_with_input', None)
+      patcher.assert_called_once_with('get', 'no_args_with_input', {})
 
   def test_hits_two_args_with_input(self):
     expected = 'works'
     with mock.patch.object(self.client, '_makeHttpRequest') as patcher:
       patcher.return_value = expected
 
-      actual = self.client.two_args_with_input('argone', arg1='argtwo')
+      actual = self.client.two_args_with_input('argone', 'argtwo', {})
       self.assertEqual(expected, actual)
 
-      patcher.assert_called_once_with('get', 'two_args_with_input/argone/argtwo', None)
+      patcher.assert_called_once_with('get', 'two_args_with_input/argone/argtwo', {})
 
   def test_input_is_procesed(self):
     expected = 'works'
@@ -327,14 +295,26 @@ class TestMakeApiCall(ClientTest):
     with mock.patch.object(self.client, '_makeHttpRequest') as patcher:
       patcher.return_value = expected
 
-      actual = self.client.no_args_with_input(payload=expected_input)
+      actual = self.client.no_args_with_input(expected_input)
       self.assertEqual(expected, actual)
 
       patcher.assert_called_once_with('get', 'no_args_with_input', expected_input)
 
-  def test_missing_input_raises(self):
+  def test_kwargs(self):
+    expected = 'works'
+    with mock.patch.object(self.client, '_makeHttpRequest') as patcher:
+      patcher.return_value = expected
+
+      actual = self.client.two_args_with_input({}, arg0='argone', arg1='argtwo')
+      self.assertEqual(expected, actual)
+
+      patcher.assert_called_once_with('get', 'two_args_with_input/argone/argtwo', {})
+
+  def test_mixing_kw_and_positional_fails(self):
     with self.assertRaises(exc.TaskclusterFailure):
-      self.client.no_args_with_input({'malformed': 'payload'})
+      self.client.two_args_no_input('arg1', arg2='arg2')
+
+  def test_missing_input_raises(self):
     with self.assertRaises(exc.TaskclusterFailure):
       self.client.no_args_with_input()
 
@@ -384,9 +364,14 @@ class TestTopicExchange(ClientTest):
 
 
 class TestBuildUrl(ClientTest):
-  def test_build_url(self):
+  def test_build_url_positional(self):
     expected = 'https://localhost:8555/v1/two_args_no_input/arg0/arg1'
-    actual = self.client.buildUrl('two_args_no_input', 'arg0', arg1='arg1')
+    actual = self.client.buildUrl('two_args_no_input', 'arg0', 'arg1')
+    self.assertEqual(expected, actual)
+
+  def test_build_url_keyword(self):
+    expected = 'https://localhost:8555/v1/two_args_no_input/arg0/arg1'
+    actual = self.client.buildUrl('two_args_no_input', arg0='arg0', arg1='arg1')
     self.assertEqual(expected, actual)
 
   def test_fails_to_build_url_for_missing_method(self):
@@ -407,22 +392,21 @@ class TestBuildSignedUrl(ClientTest):
     timePatch.return_value = 1
     self.addCleanup(timePatch.stop)
 
-  def test_builds_surl(self):
+  def test_builds_surl_positional(self):
     expBewit = 'Y2xpZW50SWRcOTAxXENVUHFtY1lSeW5Ua' + \
                '3NBS1BDaTJLUm5palgwR3hpWjFRUE9rMF' + \
                'Viamc2U1U9XGUzMD0='
     expected = 'https://localhost:8555/v1/two_args_no_input/arg0/arg1?bewit=' + expBewit
-    actual = self.client.buildSignedUrl('two_args_no_input', 'arg0', arg1='arg1')
+    actual = self.client.buildSignedUrl('two_args_no_input', 'arg0', 'arg1')
     self.assertEqual(expected, actual)
 
-
-class TestSlugId(base.TCTest):
-  def test_slug_id(self):
-    with mock.patch('uuid.uuid4') as p:
-      p.return_value = uuid.UUID('bed97923-7616-4ec8-85ed-4b695f67ac2e')
-      expected = 'vtl5I3YWTsiF7UtpX2esLg'
-      actual = subject.slugId()
-      self.assertEqual(expected, actual)
+  def test_builds_surl_keyword(self):
+    expBewit = 'Y2xpZW50SWRcOTAxXENVUHFtY1lSeW5Ua' + \
+               '3NBS1BDaTJLUm5palgwR3hpWjFRUE9rMF' + \
+               'Viamc2U1U9XGUzMD0='
+    expected = 'https://localhost:8555/v1/two_args_no_input/arg0/arg1?bewit=' + expBewit
+    actual = self.client.buildSignedUrl('two_args_no_input', arg0='arg0', arg1='arg1')
+    self.assertEqual(expected, actual)
 
 
 class TestAuthenticationMockServer(base.TCTest):
@@ -526,7 +510,7 @@ class ProductionTest(base.TCTest):
     self.assertEqual(result['alive'], True)
 
   def test_listnamespace(self):
-    result = self.i.listNamespaces('')
+    result = self.i.listNamespaces('', {})
     assert 'namespaces' in result
 
   def test_insert_to_index(self):
@@ -536,5 +520,5 @@ class ProductionTest(base.TCTest):
       'data': {'test': 'data'},
       'expires': '2015-09-09T19:19:15.879Z'
     }
-    result = self.i.insertTask('testing', payload=payload)
+    result = self.i.insertTask('testing', payload)
     self.assertEqual(payload['expires'], result['expires'])
